from __future__ import absolute_import
from sys import platform as _platform

def platform():
    """Returns whether it is running on `linux`, `osx`, or `win`."""
    if _platform == "linux" or _platform == "linux2":
        return 'linux'
    elif _platform == "darwin":
        return 'osx'
    elif _platform == "win32":
        return 'win'
    return 'unknown'



from __future__ import absolute_import
import imp
import inspect
import re

def _fetch_entity(script_filepath, regex, entity, def_within=True):
    mod_name = inspect.getmodulename(script_filepath)
    mod = imp.load_source(mod_name, script_filepath)

    funcs = []
    for func in inspect.getmembers(mod, entity):
        if def_within:
            if mod_name != func[1].__module__:
                continue
        func_name = func[0]
        m = re.match(regex, func_name)
        if m:
            funcs.append(func[1])

    return funcs


def fetch_functions(script_filepath, regex):
    """Fetches functions from a file.

    :param str script_filepath: file path.
    :param str regex: regular expression for matching function names.
    :returns: a list of functions.
    """
    return _fetch_entity(script_filepath, regex, inspect.isfunction)

def fetch_classes(script_filepath, regex):
    """Fetches classes from a file.

    :param str script_filepath: file path.
    :param str regex: regular expression for matching class names.
    :returns: a list of classes.
    """
    return _fetch_entity(script_filepath, regex, inspect.isclass)



from unicodedata import lookup as _lup

alpha = _lup("GREEK SMALL LETTER ALPHA")
beta = _lup("GREEK SMALL LETTER BETA")
gamma = _lup("GREEK SMALL LETTER GAMMA")
delta = _lup("GREEK SMALL LETTER DELTA")
epsilon = _lup("GREEK SMALL LETTER EPSILON")
zeta = _lup("GREEK SMALL LETTER ZETA")
eta = _lup("GREEK SMALL LETTER ETA")
theta = _lup("GREEK SMALL LETTER THETA")
iota = _lup("GREEK SMALL LETTER IOTA")
kappa = _lup("GREEK SMALL LETTER KAPPA")
mu = _lup("GREEK SMALL LETTER MU")
nu = _lup("GREEK SMALL LETTER NU")
xi = _lup("GREEK SMALL LETTER XI")
omicron = _lup("GREEK SMALL LETTER OMICRON")
pi = _lup("GREEK SMALL LETTER PI")
rho = _lup("GREEK SMALL LETTER RHO")
fsigma = _lup("GREEK SMALL LETTER FINAL SIGMA")
sigma = _lup("GREEK SMALL LETTER SIGMA")
tau = _lup("GREEK SMALL LETTER TAU")
upsilon = _lup("GREEK SMALL LETTER UPSILON")
phi = _lup("GREEK SMALL LETTER PHI")
chi = _lup("GREEK SMALL LETTER CHI")
psi = _lup("GREEK SMALL LETTER PSI")
omega = _lup("GREEK SMALL LETTER OMEGA")
nsum = _lup("N-ARY SUMMATION")
bone = _lup("mathematical bold digit one")

if __name__ == '__main__':
    for v in sorted(dir()):
        if v[0] != '_':
            print("%s: %s" % (v, eval(v)))



from __future__ import division
from __future__ import unicode_literals

import numpy as np
import math

def array2string(a):
    """Represents a `numpy.ndarray` as a string, in a summarized manner."""
    a = np.asarray(a)
    if len(a) <= 4:
        return np.array2string(a, 43)
    left = np.array2string(a[0:2], 25)
    left = left[:-1]
    right = np.array2string(a[-2:], 25)
    right = right[2:]

    return left + ' ... ' + right

def summarize(s, n=64):
    """Summarizes a long message."""
    assert n > 6
    s = str(s)
    if len(s) < n:
        return s

    return s[:int(n/2)-2] + ' ... ' + s[-int(math.ceil(n/2))+3:]

def make_sure_unicode(msg):
    if isinstance(msg, bytes):
        return msg.decode()
    return u"%s" % __builtins__['str'](msg)

if __name__ == '__main__':
    print(summarize('danilo horta danilo horta danilo horta danilo horta'))



from collections import OrderedDict

def _fetch_all_group_values(olist, group_name):
    values = set()
    for o in olist:
        values.add(getattr(o, group_name))
    return list(values)

def group_by(olist, group_names, sort_key=None):
    """Groups list elements."""
    if not isinstance(group_names, list):
        group_names = [group_names]

    if len(group_names) == 0:
        return olist

    group_name = group_names[0]
    vals = _fetch_all_group_values(olist, group_name)
    vals.sort(key=sort_key)
    grouped = OrderedDict([(v, []) for v in vals])

    for o in olist:
        v = getattr(o, group_name)
        grouped[v].append(o)

    for (k, tl) in grouped.iteritems():
        grouped[k] = group_by(tl, group_names[1:])

    return grouped

def traverse_dict(d, visit_func, opt=None):
    """Recursively visits each dict value."""
    opts = visit_func(d, opt)
    if isinstance(d, dict):
        i = 0
        for v in d.itervalues():
            traverse_dict(v, visit_func, opts[i] if opts else None)
            i += 1
